;                       _______  _______
;                      |       \/       |
;                RA2 --+ 1           18 +-- RA1
;                      |                |     
;                RA3 --+ 2           17 +-- RA0
;                      |                |
;               RTCC --+ 3         >>16 +-- OSC1/CLKIN
;                      |                |
;              !MCLR --+ 4 <<        15 +-- OSC2/CLKOUT
;                      |                |
;                Vss --+ 5 <<      >>14 +-- Vdd
;                      |                |
;                RB0 --+ 6         >>13 +-- RB7  DATA IN/OUT
;                      |                |
;                RB1 --+ 7         >>12 +-- RB6 
;                      |                |
;                RB2 --+ 8         >>11 +-- RB5  SCL
;                      |                |
;                RB3 --+ 9         >>10 +-- RB4  SDA
;                      |                |
;                      +----------------+
;
;******************************************************

KENWOOD	EQU	0

SDA	EQU	4
SCL	EQU	5
I_O	EQU	7
TEST	EQU	0
;******************************************************

            LIST      P=16F84, F=INHX8M
		include "P16F84.inc"

	IF KENWOOD
TIM_PAR  EQU	01BH
	ELSE
TIM_PAR  EQU	01AH
	ENDIF
;******************************************************

            ORG     0x0000

            CALL    Std_pause           ; Vuelve con W=0
            TRIS    PORTA               ;\ Define PORT A I/O
            ADDLW   0x7F
            OPTION                      ; Load OPTION register => 0

;Lee los bytes de ATR almacenados en la eeprom del pic en orden inverso, comenzando en la posicin
; 0x0E un total de 0x10 bytes

            MOVLW   0x0E                ; ATR send
            MOVWF   0x10                ; length
ATR_loop    MOVF    0x10,W              ;\
            CALL    Rd_eeprom           ;
            CALL    Send                ;   ATR
            DECFSZ  0x10,f              ;
            GOTO    ATR_loop            ;/

;Enva un 9000, final de respuesta cuando no ha habido error o un 900D cuando se simula un error
;Si el bit 3 de PCLATH vale 1 se devuelve un 900D y si vale 0 se devuelve un 9000
;La respuesta 900D se devuelve cuando se recibe un comando C140 qu eno contiene claves porque reduce el
;trfico de mensajes entre cam y card
TX9000      MOVLW   0x90                ;\
            CALL    Send
TX00_0D     MOVLW   0x00                ; 9000
            BTFSC   PCLATH,03           ;
            MOVLW   0x1D                ; 901D    
            CALL    Send

;Recibe el primer byte del comando (C1)
Recibe_C1   CALL    Receive             ; gubi komende C1 a powinien sprawdzic
                                        ; DOPISAC!
;Va a leer los 4 bytes restantes de cabecera del comando a partir de la posicin 0x0C de la RAM
;Estarn por tanto:
;  0x0C: comando   0x0D: P1    0x0E: P2     0x0F: Longitud
            MOVLW   0x0C                ;\ ustaw wskaznik bufora
            MOVWF   FSR                 ;/

Com_loop    CALL    Receive             ;\
            MOVWF   INDF
            INCF    FSR,f               ; pobierz 4 bytes komendy
            BTFSS   FSR,04              ; 0CH, 0DH, 0EH, 0FH
            GOTO    Com_loop            ;/ 

;Aqui coge el comando almacenado en la posicin 0x0D de RAM y va leyendo de la eeprom los valores de
;la lista de comandos vlidos almacenados a partir de la posicin 0x0E (EEPROM del PIC). En la
;posicin 0x10 contiene el nmero de orden del comando. 

            MOVLW   0x0D                ; Hay 0x0d comandos
            MOVWF   0x10                ; contador de comando
Comm_scan   MOVF    0x10,W
            ADDLW   0x0E                ; offset w internal EEPROM  
            CALL    Rd_eeprom           ; Lee el valor del comando
            XORWF   0x0C,W
            BTFSC   STATUS,Z            ; Si es el que se ha recibido
            GOTO    Send_ack            ; salta a procesarlo
            DECFSZ  0x10,f              ;\ Decrementa la variable del bucle
            GOTO    Comm_scan           ;/ y si no es cero vuelve a hacer otra iteracin
            MOVLW   0x6D                ;Si no ha encontrado el comando devuelve un 6D00
            CALL    Send
            GOTO    TX00_0D

Send_ack    MOVF    0x0C,W              ;\ ACK 
            CALL    Send                ;/
            MOVF    0x10,W
            ADDLW   0x2B                ;\2B es la posicin del MOVVWF. 
            CLRF    PCLATH
            MOVWF   PCL                 ;/ Desde aqui se produce un salto al goto que procesa el comando
            GOTO    Instr_0E
            GOTO    Instr_16
            GOTO    Instr_12
            GOTO    Instr_34
            GOTO    Instr_32
            GOTO    Instr_20
            GOTO    Instr_30
            GOTO    Instr_21
            GOTO    Instr_3A
            GOTO    Instr_3C
            GOTO    Instr_40
            GOTO    Instr_1A


;La instr 5A devuelve los bytes alcacenados en la EEPROM externa a partir de 0x800. Como la memoria es una
;24LC32 y slo tiene 0x800 bytes, en realidad se devuelve el contenido de la posicin 0x00. El nmero de bytes
;que se devuelve es el valor que est almacenado en la pos. 0x0F de RAM.

Instr_5A    BSF     PCLATH,03       

;*************************************************
; function : 0E C1 CLASS instruction             *
; request smartcard serial number                *
; C1 0E 00 00 08                                 *
; 0E 00 00 00 00 00 12 34 56 90 00               *
; Serial number: 001.193.046                     *
;*************************************************
;Idem a la anterior. Devuelve la cadena almacenada enla pos. 0x00 de la EEPROM externa
Instr_0E    MOVLW   0x00            ; adres w EEPROM
COMM_EEPR   CALL    EEPROM_send
            GOTO    TX9000


;Instr. 1A: Devuelve la cadena almacenada en lapos. 0x1A6 de la EEPROM externa
Instr_1A    INCF    PCLATH,f
            MOVLW   0xA6
            GOTO    COMM_EEPR

;Instr. 16: Devuelve la cadena almacenada en la pos 0x08 de la EEPROM externa
;*************************************************
; function : 16 C1 CLASS instruction             *
; ENTITIES SUPPORTED
;*************************************************
Instr_16    MOVLW   0x08            ; adres w EEPROM
            GOTO    COMM_EEPR

;****************************************************************************************
; function : 12 C1 CLASS instruction                                                    *
; this instruction requests an ID CAM                                                   *
; C1 12 aa 00 18                                                                        *
; aa is the ID provider                                                                 *
; 12 ii ii bb bb bb bb bb bb bb bb bb bb bb bb bb bb bb bb 00 xx xx yy zz zz 90 00      *
; ii ii  : provider channel                                                             *
; bb..bb : provider name                                                                *
; xx xx  : smartcard address                                                            *
; yy     : ??                                                                           *
; zz zz  : termination date  ( 7 bits year since 1990 + 4 bits month + 5 bits day       *
; data received 17H 18H 19H 1AH                                                         *
;                                                                                       *
;****************************************************************************************
;Instr. 12: En la pos 0x0D est P1, es decir, el nmero de proveedor.
;Inicializa w con -8 (0xF8) y cada pasada del bucle incrementa W en 0x18 que son los bytes
;que ocupa el ID CAM de cada proveedor. Puesto que el bucle comienza incrementando W
;los datos del primer proveedor estn almacenados en la posicin 0x18-0x08=0x10 de la
;EEPROM externa. Hay una condicin poara que cuando se llegue a la posicin 0x100 incrementar
;PCLATH. Por tanto al final del bucle PCLATH contiene la parte alta de la direccin de donde leer los
;datos y W la parte baja.
Instr_12    MOVLW   0xF8           ; offset w EEPROM
            INCF    0x0D,f         ; zwieksz wskaznik
instr_12_loop  
            ADDLW   0x18           ; 
            BTFSC   STATUS,Z       ; Skip if not zero
            INCF    PCLATH,f
            DECFSZ  0x0D,f
            GOTO    instr_12_loop
            GOTO    COMM_EEPR

;*************************************************
; function : 3A C1 CLASS instruction             *
; request for the control word                   *
;C1 
;3A 00 00 10 
;3A 					; ACK
;10 10 55 E6 55 54 05 80 
;04 14 40 8F 40 41 11 23 
;90 00 
;*************************************************
;Esta instruccin pide las control words desencriptadas que se enviaron anteriormente mediante
;una instr c13c. Los valores desencriptados se han dejado almacenados a partir de la pos 0x30 de
;la RAM. Hay un bucle que va leyendo a travs del registro indirecto los valores y envindolos
Instr_3A    MOVLW   0x30
            MOVWF   FSR               ; wskaznik bufora w RAM
instr_3A_loop  
            MOVF    INDF,W            ; wez z bufora
            CALL    Send              ; wyslij
            INCF    FSR,f             ; zwieksz wskaznik
            DECFSZ  0x0F,f            ; odlicz dlugosc
            GOTO    instr_3A_loop
            GOTO    TX9000            ; zakoncz

;*************************************************
; function : 3C C1 CLASS instruction             *
; ENTITLEMENT CONTROL                            *
;C1 
;3C 						; komenda
;04 						; 4-ty provider na karcie,
;0C 						; klucz 0C
;27		 				; 27H bytes long
;3C						; ACK
;71 00 						; unknown
;00 10 						; provider
;07 D0 						; year
;00 08						; month
;27 						; separator
;15 0C 						; complete data
;13 21 						; boundle
;[D1]						; marker
;EF 44 F8 57 15 A0 A0 A6			; control word 1
;D0 11 1C 81 87 6E 24 8E			; control word 2
;82 						; separator
;23 C0 DF FB 10 19 62 DB 			; sygnatura
;90 00 
;*************************************************
;Esta instruccin envia control words encriptados para que se desencripten

; Primero se llema a GetCW. Esta rutina recibe el resto del comando,
;lee la key de 16 bytes desde la EEPROM externa segn los valores de P1 y P2 (en 0x0d y 0x0e) 
;y en caso de que haya superencriptacin realiza la primera desencriptacin
Instr_3C    CALL    Get_CW         ; tu mozna ew. dopisac gaszenie LED'a
				           ; ID providera jest w 14h

;Aqui estn las rutinas xor___ cuyo nombre est mal. La rutina xor_eedat hace un swap de un 
;bloque de 8 bytes con otro. En W va la posicin de incio del bloque y en EEADR la distancia que hay que moverlo
;en 0x14 Get_CW ha almacenado la siguiente posicin libre tras el comando almacenado
            MOVF    0x14,W
            SUBLW   0x1D
            MOVWF   EEADR
            MOVLW   0x0C
            CALL    xor_eedat    ;Swap del bloque que comienza 0x11 bytes antes del final a 0x0c (pone 0x1d porque es 0x11+0xc)
                                 ;Es el segundo bloque encriptado
            CALL    xor_2C_38    ;Swap de los bytes de 0x38-0x3f con 0x0c-0x13
            MOVF    0x14,W         ;\   
            SUBLW   0x25         ;Swap del bloque que comienza 0x19 bytes antes del final a 0x0c (pone 0x25 porque es 0x19+0xc)
            MOVWF   EEADR
            MOVLW   0x0C           ;/
            CALL    xor_eedat
            CALL    xor_24_30    ;Swap de 0x30-0x37 con 0x0c-0x13
            CALL    Encrypt      ;Desencripta bloque
            CALL    xor_F8_30    ;Swap de 0x30-0x37 con 0x38-0x3f
            CALL    Encrypt      ;Desencripta
            CALL    xor_F8_30    ;Swap de 0x30-0x37 con 0x38-0x3f
            GOTO    TX9000
;----------------------------------------------------
Get_CW      MOVF    0x0F,W        ; Lee la longitud del comando
            MOVWF   0x10          ; y la almacena en la pos 0x10 de RAM
GET_CW_loop MOVLW   0x31
            SUBWF   0x10,W
            BTFSS   STATUS,C      ; Skip if carry set
            GOTO    c068c
            MOVLW   0x08          ;Si el comando tiene ms de 0x31 bytes
            SUBWF   0x10,f	    ; lee y descarta 8 bytes. 
            MOVWF   0x0F          
            CALL    Doczytaj
            GOTO    GET_CW_loop

c068c       MOVLW   0x18          ;\ Almacena el resto del comando a partir de la pos. 0x18 de RAM
            MOVWF   FSR           ;/
c068c_loop  CALL    Receive       ;\
            MOVWF   INDF
            INCF    FSR,f
            DECFSZ  0x10,f
            GOTO    c068c_loop    ;/

            MOVF    FSR,W
            MOVWF   0x14          ; En 0x14 almacena la siguiente posicin libre tras el comando
            CALL    c112c         ; Calcula la dir en EEPROM donde esta la key a utilizar
            MOVLW   0x40          ;Las keys se leen a partir de la pos 0x40. Por eso el comando no puede tener
            MOVWF   FSR           ;ms de 0x31 bytes, para asegurar que no es sobreescrito por las keys 
            CALL    Get_key       ; obtiene key de 8 bytes de la eeprom
            MOVLW   0x40
            BTFSC   0x0D,03       ;Si se utilizan keys secundarias
            ADDWF   0x10,f        ;Coge la secundaria 0x40 bytes ms adelante (siguiente provider) 
            CALL    Get_key       ; obtiene los otros 8 bytes de la key
            BTFSS   PCLATH,04
            GOTO    c08Dc         ;Si no hay superencriptacin se sale
            MOVLW   0x18          ;Pero si la hay se desencripta toda la instruccin: 
            CALL    xor_xx_30     ;Swap de 0x30-0x37 con 0x18-0x2f
            CALL    Encrypt       ;Desencripta 
            MOVLW   0x18          ;Swap de 0x30-0x37 con 0x18-0x1f
            CALL    xor_xx_30
            MOVLW   0x10          ;Swap de 0x30-0x37 con 0x20-0x27
            CALL    xor_xx_30
            CALL    Encrypt       ;Desencripta
            MOVLW   0x10
            CALL    xor_xx_30     ;Swap de 0x30-0x37 con 0x20-0x27
            MOVLW   0x08
            CALL    xor_xx_30     ;Swap de 0x30-0x37 con 0x28-0x2f
            CALL    Encrypt       ;Desencripta
            MOVLW   0x08
            CALL    xor_xx_30     ;Swap de 0x30-0x37 con 0x28-0x2f
            CALL    c08Bc         ;Swap de 0x30-0x37 con 0x38-0x3f, desencripta
c08Bc       CALL    xor_F8_30     ;otra vez  
            CALL    Encrypt     
c08Dc       RETURN                


;Hace un swap del bloque 0x30-0x37 con el 0x0c-0x13
xor_F8_30   MOVLW   0xF8
xor_xx_30   MOVWF   EEADR         ;;Swap de 30 con la diferencia indicada en W
            MOVLW   0x30
            CALL    xor_eedat
            RETURN

;Rutina de desencriptacin SECA
;0x0F apunta al elemento de la key que se est procesando (0x00..0x0f)
;0c0D es la constante C
Encrypt     CLRF    0x0F
            MOVLW   0x02
            MOVWF   PCLATH   ;Las key se leen de la pgina 4 de EEPROM
            CLRF    0x0D
;Preparacin de la key
Encrypt_loop  
            CALL    c0FFc    ;Hace K[i]=K[i] XOR T1[K[i-1] XOR K[i+1] XOR C] (i es el valor de 0x0F)
            INCF    0x0F,f   ;i=i+1
            MOVF    0x0F,W
            ANDLW   0x03     
            BTFSC   STATUS,Z              
            INCF    0x0D,f   ;Cada 4 pasadas del bucle incrementa la constante C
            BTFSS   0x0F,06  
            GOTO    Encrypt_loop  ;Hacer 16 pasadas
;Paso 1: XOR key bytes con data bytes. Hace tambin los lookups iniciales del paso 2
            MOVLW   0x0F
            MOVWF   0x0D     ;Constante C vale inicialmente 0x0f
            MOVLW   0x0C     
            MOVWF   0x0F     ;i=13
c0A3c       CLRW
            CALL    c10Cc    ;Lee K[i] a W
            XORWF   0x34,W   ;W=d[4] XOR K[i]
            CALL    Tablica  
            MOVWF   0x10     ;[0x10]=T1[W]
            MOVLW   0x01
            CALL    c10Cc    ;w=d[4+1]=d[5]
            XORWF   0x35,W   ;W=d[5] XOR K[i]
            CALL    Tablica
            MOVWF   0x11     ;[0x11]=T1[W]
            MOVLW   0x02     
            CALL    c10Cc
            XORWF   0x36,W
            CALL    Tablica
            MOVWF   0x12     ;[0x12]=T1[d[6] XOR K[i]]
            MOVLW   0x03
            CALL    c10Cc
            XORWF   0x37,W
            CALL    Tablica
            MOVWF   0x13     ;[0x13]=T1[d[7] XOR K[i]] 
;Paso 2: Core function. El paso a (T1 table lookup) ya se hizo en el paso anterior
;Los nuevos valores de d[4]..d[7] estn almacenados en 0x10..0x12
;Al final del paso 1 en W est almacenado d[7]
            INCF    PCLATH,f  ;Apunta a T2 en la EEPROM externa
            XORWF   0x10,f    ;d[4]=d[4] XOR d[7]
            SWAPF   0x13,W    ;Swap nibbles de d[7]
            ADDWF   0x12,W    ;W=~d[7]+d[6] 
            CALL    c1ADc     ;W=T2[W]
            MOVWF   0x12      ;d[6]=T2[~d[7]+d[6]]
            XORWF   0x13,f    ;d[7]=d[7] XOR d[6]
            SWAPF   0x12,W
            ADDWF   0x11,W
            CALL    c1ADc
            MOVWF   0x11      ;d[5]=T2[~d[6]+d[5]]
            XORWF   0x12,f    ;d[6]=d[6] XOR d[5]
            SWAPF   0x11,W
            ADDWF   0x10,W
            CALL    c1ADc
            MOVWF   0x10      ;d[4]=T2[~d[5]+d[4]]
            DECF    PCLATH,f  ;Vuelve a apuntar a las keys
            XORWF   0x11,f    ;d[5]=d[5] XOR d[4] 
            SWAPF   0x10,W       
            ADDWF   0x13,W    
            CALL    Tablica
            MOVWF   0x13      ;d[7]=T1[~d[4]+d[7]]
            XORWF   0x10,f    ;d[4]=d[4] XOR d[7]
            SWAPF   0x13,W
            ADDWF   0x12,W
            CALL    Tablica
            MOVWF   0x12      ;d[6]=T1[~d[7]+d[6]]
            XORWF   0x13,f    ;d[7]=d[7] XOR d[6]
            SWAPF   0x12,W
            ADDWF   0x11,W
            CALL    Tablica
            MOVWF   0x11      ;d[5]=T1[~d[6]+d[5]]
            XORWF   0x12,f    ;d[6]=d[6] XOR d[5]
            MOVF    0x10,W
            XORWF   0x11,f    ;d[5]=d[5] XOR d[4]
;Paso 3: Obtener valores de datos para la siguiente iteracin. 
            INCF    PCLATH,f  ;Apunta a T2
            MOVF    0x11,W        
            CALL    c1ADc
            XORWF   0x30,f    ;d[0]=d[0] XOR T2[d[5]]
            MOVF    0x13,W   
            CALL    c1ADc
            XORWF   0x31,f    ;d[1]=d[1] XOR T2[d[7]]
            MOVF    0x10,W
            CALL    c1ADc
            XORWF   0x32,f    ;d[2]=d[2] XOR T2[d[4]] 
            MOVF    0x12,W
            CALL    c1ADc
            XORWF   0x33,f    ;d[3]=d[3] XOR T2[d[6]]
            DECF    PCLATH,f  ;Apunta a keys
;Al final no se hace el swap de d[0]..d[3] con d[4]..d[7]. Se deja para ms adelante cuando se sepa si hay que
;iterar otra vez
;Paso 4: Preparar keys para la siguiente iteracin
            MOVLW   0x03        
            ADDWF   0x0F,f    
            CALL    c0FFc     ;K[i+3]=K[i+3] XOR T1[K[i+4] XOR K[i-2]]
            DECF    0x0F,f    
            CALL    c0FFc     ;K[i+2]=K[i+2] XOR T1[K[i+3] XOR K[i-1]]
            DECF    0x0F,f
            CALL    c0FFc     ;K[i+1]=K[i+1] XOR T1[K[i+2] XOR K[i]]
            DECF    0x0F,f
            CALL    c0FFc     ;K[i]=K[i] XOR T1[K[i+1] XOR K[i-1]]
            MOVF    0x0D,f
            BTFSC   STATUS,Z  ;Si C=0 retornar
            RETURN
;Paso 3.b. Hay que iterar otra vez: Hacer swap y decrementar C
            MOVLW   0x04      
            MOVWF   EEADR
            MOVLW   0x34
            CALL    c199c     ;Swap de d[0]..d[3] con d[4]..d[7]
            DECF    0x0D,f    ;C=C-1
            MOVLW   0xFC      
            ADDWF   0x0F,f    ;i=i-4 
            MOVLW   0x0C
            BTFSC   0x0F,07   ;Si i se ha vuelto negativo
            MOVWF   0x0F      ;lo pone de nuevo a 0x0C
            GOTO    c0A3c

;Realiza la operacin K[i]=K[i] XOR T1[K[i-1] XOR K[i+1] XOR C]
;en 0x0f est contenido el valor de i y en 0x0D el de C
c0FFc       MOVLW   0x01
            CALL    c10Cc   ;Lee a W K[i+1]
            MOVWF   0x0C    ;Y lo almacena temporalmente en 0x0C 
            MOVLW   0xFF    
            CALL    c10Cc   ;Lee a W K[i-1]
            XORWF   0x0C,f  ;Hace K[i-1] XOR K[i+1]
            CLRW
            CALL    c10Cc   ;Lee K[i]. No usa el resultado, pero consigue que FSR apunte a K[i]
            MOVF    0x0C,W  
            XORWF   0x0D,W  ;Hace el XOR C
            CALL    Tablica ;Hace el T1[....]
            XORWF   INDF,f  ;Y por ltimo K[i]=K[i] XOR ...
            RETURN

;Rutina que devuelve el elemento situado a un desplazamiento W del actualmente apuntado en 0x0F
;Se utiliza para obtener un elemento de la key relativo al actualmente apuntado por 0x0F
;La rutina controla el wrap
c10Cc       ADDWF   0x0F,W   ;Aade el desplazamiento a la posicin contenida en 0x0f y almacena el resultado en W
            ANDLW   0x0F     ;Controla el wrap. El elemento siguiente al 0xf es el 0 otra vez
            MOVWF   FSR      
            BSF     FSR, 06  ;La key est almacenada a partir de 0x40
            MOVF    INDF,W   ;Lee el valor
            RETURN
;---------------
; Calcula el offset en la eeprom externa de la key a utilizar
;Devuelve en 0x10 el offset de la key y en PCLATH la pgina. El bit 4 de PCLATH se activa para indicar superencriptacin
c112c       CLRF    0x10       
            RRF     0x0D,f       ;Pasa los 2 bytes bajos del num. de provider a los altos de 0x10
            RRF     0x10,f       ;Equivalente a desplazar a la izda 6 bits. Esto es porque cada provider ocupa 64 bytes (2^6)
            RRF     0x0D,W       ;W contiene el los bits 1-4 del provider
            RRF     0x10,f
            ANDLW   0x03         ;Se queda con los bits 2-3 del provider
            MOVWF   PCLATH       ;que marcan la pgina (en una pgina (256 bytes) caben 4 providers (4 prov*8 keys*8 bytes/key)
            BSF     PCLATH,02    ;Las keys comienzan en 0x400 (pag 4)
            BTFSC   0x0E,07      ;Si hay superencriptacion
            BSF     PCLATH,04    ;Activa el bit 4 de PCLATH
            MOVLW   0x07         
            ANDWF   0x0E,f       ;Coge el nmero de la key
c11Ec       MOVLW   0x08         ;Cada key ocupa 8 bytes
            DECF    0x0E,f
            BTFSC   0x0E,07      ;Mientras sea positivo, continua el bucle (acaba con el valor -1(0xff))
            RETURN

            ADDWF   0x10,f
            GOTO    c11Ec
;-------------------------------------
; Lee una key de la EEPROM externa. El offset viene en la pos 0x10 de RAM. Se almacena donde indica el registro indirecto
Get_key     MOVF    0x10,W
; Idem a la anterior pero el offset viene en el registro W
c125c       CALL i2c_wr                ; Enva direccin a leer
            MOVLW   0x08               ; Leer 8 bytes
            MOVWF   0x11               ; 0x11 es el contador del bucle
c125c_loop  CALL    i2c_rd
            MOVWF   INDF
            INCF    FSR,f
            DECFSZ  0x11,f
            GOTO    c125c_loop
            RETURN

;*************************************************
; function : 34 C1 CLASS instruction             *
; request for the control word                   *
;*************************************************
Instr_34    CALL    Doczytaj   ;Ignora los bytes recibidos
            GOTO    TX9000     ;Y devuelve OK

;*************************************************
; function : 32 C1 CLASS instruction             *
; extra and general infos                        *
;*************************************************
Instr_32    INCF    PCLATH,f   ;Si la longitud es impar enva la cadena de la EEPROM externa contenida en 0x199   
            MOVLW   0x99       ;porque supone que es el comando de Startup Record
            BTFSS   0x0F,00  
            GOTO    c3A6c      ;Si es par salta porque supone que es el comando de extra y general infos
            GOTO    COMM_EEPR

;*************************************************
; function : 20 C1 CLASS instruction             *
; pobranie danych do bufora i zapis do EEPROM    *
;*************************************************
Instr_20    MOVF    0x0F,W          ;No hace nada. Est de versiones anteriores del cdigo
                                    ; 
            MOVLW   0x38            ;\ Almacena el bloque de datos a enviar a partir de la pos 0x38
            MOVWF   FSR             ;/ 
instr_20_loop  
            CALL    Receive         ;\ Recibe un byte
            MOVWF   INDF            ;  y lo almacena a travs del registro indirecto
            INCF    FSR,f           ; Apunta al siguiente byte   
            BTFSS   FSR,06          ; hasta completar 64 bytes
            GOTO    instr_20_loop   ;/
            MOVF    0x0D,W          ; P1= parte alta de la direccin de la EEPROM externa
            ADDWF   PCLATH,f
            MOVF    0x0E,W          ; P2= parte baja
            CALL    i2c_wr          ; Envia la direccin de escritura a la EEPROM externa
            CALL    wr_i2c_string   ; y escribe los datos
            GOTO    TX9000

;---------------------------------------------------
wr_i2c_string  
            MOVLW   0x37            ;\ La cadena est almacenada a partir de 0x38. Pone 0x37 porque el bucle comienza incrementando la posicin
            MOVWF   FSR             ;/ ustaw bufor
wr_i2c_str_loop  
            INCF    FSR,f
            MOVF    INDF,W
            CALL    wystaw_byte     ;Enva byte 
            DECFSZ  0x0F,f          ;Decrementa el nmero de bytes que quedan por enviar  
            GOTO    wr_i2c_str_loop
            GOTO    i2c_stop        ;Fin de la escritura 
;---------------------------------------------------
c14Dc       SUBWF   0x0F,f            ;Recibe y descarta el nmero de bytes contenidos en la pos 0x0f de RAM menos los bytes indicados por W 
Doczytaj    CALL    Receive        ;\ ;Recibe y descarta el nmero de byten contenidos en la pos 0x0f de RAM
            DECFSZ  0x0F,f
            GOTO    Doczytaj       ;/
            RETURN

;========================================
; procedura wyslania bytes na I/O karty. Dziala ale ma KNOT!
; b. ladnie rowiazane liczenie sumy kontrolnej

Send        MOVWF   0x16
            MOVLW   0x32
            CALL    Pause
            MOVLW   0x7F           ;\
            TRIS    PORTB          ;/ Define PORT B I/O RB7=output
            CLRF    0x17
            BCF     PORTB,07       ; wystaw bit startu
            MOVLW   0x08           ;\ 
            MOVWF   0x14           ;/ length ilosci bitow
send_loop   CALL    Std_pause      ; czas jednego bitu
            MOVF    0x16,W         ; pobierz dana
            XORWF   0x17,f         ; suma kontrolna
            BCF     PORTB,07
            BTFSC   0x16,00        ; skok jesli bit 16H.0 = 0
            BSF     PORTB,07
            RRF     0x16,f         ; przesun 
            DECFSZ  0x14,f
            GOTO    send_loop      ;/
            CALL    Std_pause      ;\ 
            BCF     PORTB,07       ; ustaw 0    
            BTFSC   0x17,00        ; skok jesli bit sumy kontrolnej = 0
            BSF     PORTB,07       ; ustaw 1
            CALL    Std_pause      ;/
            MOVLW   0xFF           ;\ Define PORT B I/O = input
            TRIS    PORTB          ;/
            MOVLW   0x4F           ;\ parametr czasu
            GOTO    Pause          ;/ 2 bity stopu

;-----------------------------------------------
Receive     BTFSC   PORTB,07       ; skip if I/O = 0
            GOTO    Receive        ; czekaj na bit startu
            MOVLW   0x2A           ; parametr czasu
            CALL    Pause          ; przeczekaj bit startu
            MOVLW   0x09           ; ilosc bitow
            MOVWF   0x14           ; length
Receive_loop  
            BCF     STATUS,C       ; Clear carry flag
            BTFSC   PORTB,07
            BSF     STATUS,C       ; Set carry flag
            RRF     0x16,f         ; poobracaj...
            CALL    Std_pause      ;
            DECFSZ  0x14,f
            GOTO    Receive_loop   ; petla
            RLF     0x16,f         ; ustal dane
            MOVLW   0x28           ;\ parametr czasu
            CALL    Pause          ;/ olej bit sumy i bity stopu
            MOVF    0x16,W         ; odebrany byte do W
            RETURN
;-------------------------------------------------
I2C_busy    CALL    i2c_start
            BCF     STATUS,C       ; Clear carry flag
            CALL    wystaw_adres
            BTFSC   STATUS,C       ; Skip if no busy
            GOTO    I2C_busy
            RETURN
;-------------------------------------------------
Std_pause   MOVLW   TIM_PAR        ;Pausa estndar para enviar/recibir con protocolo serie
Pause       MOVWF   0x15           ;Hace una pausa del tiempo indicado en W
Pause_loop  DECFSZ  0x15,f
            GOTO    Pause_loop
            RETLW   0x00           ;Siempre devuelve 0
;-------------------------------------------------
;Lee el valor de una posicin de la EEPROM del PIC.
;Recibe en W la direccin y develve en W el valor leido
Rd_eeprom   MOVWF   EEADR          
            BSF     STATUS,RP0    ; Select page 1 (80H - FFH)
            BSF     EECON1,00
            BCF     STATUS,RP0    ; Select page 0 (00H - 7FH)
            MOVF    EEDATA,W
            RETURN
;---------------------------------------------------
;Envia la cadena de datos contenida en la EEPROM externa a travs del puerto serie
;La direccin viene en W la parte baja y en PCLATH la alta. En 0x0F est el nmero de bytes a enviar
EEPROM_send CALL    i2c_wr        ; Enva direccin
eepr_send_loop  
            DECF    0x0F,f        ; Decrementa contador
            BTFSC   0x0F,07       ;Si es negativo ha acabado y retorna 
            RETURN

            CALL    i2c_rd        ; lee dato
            CALL    Send          ; y lo enva
            GOTO    eepr_send_loop
;--------------------------------------------------
; Rutina que hace un swap de dos bloques de 8 bytes cada uno. W contiene la direccin de incio de un bloque y eeadr
; el desplazamiento del segundo bloque respecto del primero. Si el desplazamiento es positivo significa mover
; hacia atrs y si es negativo hacia adelante.
xor_eedat   CALL    c199c       ;Swap de dos bloques de 8 bytes
            GOTO    c19Ac
c199c       MOVWF   FSR         ;Swap de dos bloques de 4 bytes
c19Ac       CALL    c19Dc
            CALL    c19Dc
            CALL    c19Dc
c19Dc       MOVF    INDF,W      ;Swap de 1 byte
            MOVWF   EEDATA
            MOVF    EEADR,W
            SUBWF   FSR,f
            MOVF    EEDATA,W
            XORWF   INDF,f
            MOVF    INDF,W
            XORWF   EEDATA,f
            MOVF    EEDATA,W
            XORWF   INDF,f
            MOVF    EEADR,W
            ADDWF   FSR,f
            MOVF    EEDATA,W
            MOVWF   INDF
            INCF    FSR,f
            RETURN
;-----------------------------------------------------
;Devuelve el valor de T2 indicado en el registro W
c1ADc       MOVWF   EEADR
            CALL    i2c_start
            BCF     STATUS,C               ; Clear carry flag
            CALL    wystaw_adres
            MOVF    EEADR,W
            CALL    wystaw_byte
;------------------------------------------------------
;Lee el byte de EEPROM externa apuntado por W y PCLATH. El valor es devuelto en W
i2c_rd      CALL    i2c_start
            BSF     STATUS,C               ; Set carry flag
            CALL    wystaw_adres
            CALL    wez_byte
            CALL    i2c_stop
            MOVF    EEDATA,W
            RETURN
;------------------------------------------------------
i2c_wr      MOVWF   EEADR
            CALL    I2C_busy
            MOVF    EEADR,W
            GOTO    wystaw_byte

wr_last     CALL    wystaw_byte
            GOTO    i2c_stop
;-----------------------------------------
wystaw_adres  
            RLF     PCLATH,W
            ANDLW   0x0F                 ; wytnij mlodsza czesc adresowa
            IORLW   0xA0                 ; dodaj adres I2C
wystaw_byte MOVWF   EEDATA
            MOVLW   0xCF                 ;\
            TRIS    PORTB                ;/ Define PORT B I/O
            CALL    wystaw_nible
            CALL    wystaw_nible
            GOTO    wez_bit
wystaw_nible  
            CALL    wystaw_nible_2
            CALL    wystaw_nible_2
            CALL    wystaw_nible_2
wystaw_nible_2  
            RLF     EEDATA,f
            GOTO    wystaw_bit_x

wez_byte    MOVLW   0xDF                 ;\
            TRIS    PORTB                ;/ Define PORT B I/O
            CALL    wez_nible
            CALL    wez_nible
            BSF     STATUS,C             ; Set carry flag
            GOTO    wystaw_bit
wez_nible   CALL    wez_nible_2
            CALL    wez_nible_2
            CALL    wez_nible_2
wez_nible_2 CALL    wez_bit_x
            RLF     EEDATA,f
            RETURN

;-------------------------------------------------
i2c_start   BSF     PORTB,04
            MOVLW   0xCF                ;\
            TRIS    PORTB               ;/ Define PORT B I/O
            BSF     PORTB,05
            NOP
            BCF     PORTB,04
            GOTO    c1E9c
;-----------------------------------------------------
i2c_stop    BCF     PORTB,04
            MOVLW   0xCF
            TRIS    PORTB               ; Define PORT B I/O
            NOP
            BSF     PORTB,05
            NOP
            BSF     PORTB,04
c1E8c       NOP
c1E9c       BCF     PORTB,05
            RETURN

wystaw_bit  MOVLW   0xCF                ;\
            TRIS    PORTB               ;/ Define PORT B I/O
wystaw_bit_x  
            BTFSS   STATUS,C            ; Skip if carry set
            BCF     PORTB,04
            BTFSC   STATUS,C            ; Skip if no carry
            BSF     PORTB,04
            BSF     PORTB,05
            GOTO    c1E8c
;----------------------------------------------------
wez_bit     MOVLW   0xDF                ;\
            TRIS    PORTB               ;/ Define PORT B I/O
wez_bit_x   BSF     PORTB,05
            NOP
            BSF     STATUS,C            ; Set carry flag
            BTFSS   PORTB,04
            BCF     STATUS,C            ; Clear carry flag
            GOTO    c1E9c

;Devuelve el valor de tabla1 del algoritmo de SECA que corresponde al valor contenido en W
;Es necesario antes de llamar a esta rutina que PCLATH valga 2 porque la tabla comienza en el offset 0x200
Tablica     MOVWF   PCL    ; Absolute addressing
;---------------------------------------------
            ORG	 0200H

            RETLW   0x2A
            RETLW   0xE1
            RETLW   0x0B
            RETLW   0x13
            RETLW   0x3E
            RETLW   0x6E
            RETLW   0x32
            RETLW   0x48
            RETLW   0xD3
            RETLW   0x31
            RETLW   0x08
            RETLW   0x8C
            RETLW   0x8F
            RETLW   0x95
            RETLW   0xBD
            RETLW   0xD0
            RETLW   0xE4
            RETLW   0x6D
            RETLW   0x50
            RETLW   0x81
            RETLW   0x20
            RETLW   0x30
            RETLW   0xBB
            RETLW   0x75
            RETLW   0xF5
            RETLW   0xD4
            RETLW   0x7C
            RETLW   0x87
            RETLW   0x2C
            RETLW   0x4E
            RETLW   0xE8
            RETLW   0xF4
            RETLW   0xBE
            RETLW   0x24
            RETLW   0x9E
            RETLW   0x4D
            RETLW   0x80
            RETLW   0x37
            RETLW   0xD2
            RETLW   0x5F
            RETLW   0xDB
            RETLW   0x04
            RETLW   0x7A
            RETLW   0x3F
            RETLW   0x14
            RETLW   0x72
            RETLW   0x67
            RETLW   0x2D
            RETLW   0xCD
            RETLW   0x15
            RETLW   0xA6
            RETLW   0x4C
            RETLW   0x2E
            RETLW   0x3B
            RETLW   0x0C
            RETLW   0x41
            RETLW   0x62
            RETLW   0xFA
            RETLW   0xEE
            RETLW   0x83
            RETLW   0x1E
            RETLW   0xA2
            RETLW   0x01
            RETLW   0x0E
            RETLW   0x7F
            RETLW   0x59
            RETLW   0xC9
            RETLW   0xB9
            RETLW   0xC4
            RETLW   0x9D
            RETLW   0x9B
            RETLW   0x1B
            RETLW   0x9C
            RETLW   0xCA
            RETLW   0xAF
            RETLW   0x3C
            RETLW   0x73
            RETLW   0x1A
            RETLW   0x65
            RETLW   0xB1
            RETLW   0x76
            RETLW   0x84
            RETLW   0x39
            RETLW   0x98
            RETLW   0xE9
            RETLW   0x53
            RETLW   0x94
            RETLW   0xBA
            RETLW   0x1D
            RETLW   0x29
            RETLW   0xCF
            RETLW   0xB4
            RETLW   0x0D
            RETLW   0x05
            RETLW   0x7D
            RETLW   0xD1
            RETLW   0xD7
            RETLW   0x0A
            RETLW   0xA0
            RETLW   0x5C
            RETLW   0x91
            RETLW   0x71
            RETLW   0x92
            RETLW   0x88
            RETLW   0xAB
            RETLW   0x93
            RETLW   0x11
            RETLW   0x8A
            RETLW   0xD6
            RETLW   0x5A
            RETLW   0x77
            RETLW   0xB5
            RETLW   0xC3
            RETLW   0x19
            RETLW   0xC1
            RETLW   0xC7
            RETLW   0x8E
            RETLW   0xF9
            RETLW   0xEC
            RETLW   0x35
            RETLW   0x4B
            RETLW   0xCC
            RETLW   0xD9
            RETLW   0x4A
            RETLW   0x18
            RETLW   0x23
            RETLW   0x9F
            RETLW   0x52
            RETLW   0xDD
            RETLW   0xE3
            RETLW   0xAD
            RETLW   0x7B
            RETLW   0x47
            RETLW   0x97
            RETLW   0x60
            RETLW   0x10
            RETLW   0x43
            RETLW   0xEF
            RETLW   0x07
            RETLW   0xA5
            RETLW   0x49
            RETLW   0xC6
            RETLW   0xB3
            RETLW   0x55
            RETLW   0x28
            RETLW   0x51
            RETLW   0x5D
            RETLW   0x64
            RETLW   0x66
            RETLW   0xFC
            RETLW   0x44
            RETLW   0x42
            RETLW   0xBC
            RETLW   0x26
            RETLW   0x09
            RETLW   0x74
            RETLW   0x6F
            RETLW   0xF7
            RETLW   0x6B
            RETLW   0x4F
            RETLW   0x2F
            RETLW   0xF0
            RETLW   0xEA
            RETLW   0xB8
            RETLW   0xAE
            RETLW   0xF3
            RETLW   0x63
            RETLW   0x6A
            RETLW   0x56
            RETLW   0xB2
            RETLW   0x02
            RETLW   0xD8
            RETLW   0x34
            RETLW   0xA4
            RETLW   0x00
            RETLW   0xE6
            RETLW   0x58
            RETLW   0xEB
            RETLW   0xA3
            RETLW   0x82
            RETLW   0x85
            RETLW   0x45
            RETLW   0xE0
            RETLW   0x89
            RETLW   0x7E
            RETLW   0xFD
            RETLW   0xF2
            RETLW   0x3A
            RETLW   0x36
            RETLW   0x57
            RETLW   0xFF
            RETLW   0x06
            RETLW   0x69
            RETLW   0x54
            RETLW   0x79
            RETLW   0x9A
            RETLW   0xB6
            RETLW   0x6C
            RETLW   0xDC
            RETLW   0x8B
            RETLW   0xA7
            RETLW   0x1F
            RETLW   0x90
            RETLW   0x03
            RETLW   0x17
            RETLW   0x1C
            RETLW   0xED
            RETLW   0xD5
            RETLW   0xAA
            RETLW   0x5E
            RETLW   0xFE
            RETLW   0xDA
            RETLW   0x78
            RETLW   0xB0
            RETLW   0xBF
            RETLW   0x12
            RETLW   0xA8
            RETLW   0x22
            RETLW   0x21
            RETLW   0x3D
            RETLW   0xC2
            RETLW   0xC0
            RETLW   0xB7
            RETLW   0xA9
            RETLW   0xE7
            RETLW   0x33
            RETLW   0xFB
            RETLW   0xF1
            RETLW   0x70
            RETLW   0xE5
            RETLW   0x17
            RETLW   0x96
            RETLW   0xF8
            RETLW   0x8D
            RETLW   0x46
            RETLW   0xA1
            RETLW   0x86
            RETLW   0xE2
            RETLW   0x40
            RETLW   0x38
            RETLW   0xF6
            RETLW   0x68
            RETLW   0x25
            RETLW   0x16
            RETLW   0xAC
            RETLW   0x61
            RETLW   0x27
            RETLW   0xCB
            RETLW   0x5B
            RETLW   0xC8
            RETLW   0x2B
            RETLW   0x0F
            RETLW   0x99
            RETLW   0xDE
            RETLW   0xCE
            RETLW   0xC5

;*************************************************
; function : 30 C1 CLASS instruction             *
; PIN CONTROLLED OPERATIONS                      *
;*************************************************
Instr_30    MOVLW   0x02
            CALL    c14Dc              ; Descarta todo salvo los 2 ltimos bytes (PIN tecleado). Sale con 0x0f=0
            CALL    Receive            ;\Recibe el MSB byte del PIN 
            MOVWF   0x0F               ;La almacena en 0x0F
            CALL    Receive            ;\Recibe el LSB byte
            MOVWF   0x0D               ;Y lo almacena en 0x0D
            MOVLW   0x1B               ;Segundo byte del pin
            XORWF   0x0F
            BTFSS   STATUS,Z
            GOTO    TX9010
            MOVLW   0x39               ;Segundo byte del pin
            XORWF   0x0D
            BTFSS   STATUS,Z
            GOTO    TX9010
            GOTO    TX9000

TX9010      MOVLW   0X90
            CALL    Send
            MOVLW   0x10
            CALL    Send
            GOTO    Recibe_C1

;**************************************************
; function :  C1 40 CLASS instruction             *
;ENTITLEMENT MANAGEMENT                           *
;**************************************************
Instr_40    MOVF    0x0D,W
            MOVWF   0x17            ;Guarda P1 en 0x17  
            BSF     PCLATH,03       ;Pgina 0x400
            MOVLW   0x34
            SUBWF   0x0F,W          ;W=Longitud instr-0x34
            BTFSS   STATUS,C        ; Skip if carry set
            GOTO    Instr_34        ;Si menor de 0x34 bytes salta a la instr. 34
            CALL    Get_CW          ;Obtiene las keys correspondientes de la EEPROM y realiza la primera desencriptacin
            BSF     PCLATH,03       ;Pgina 0x400
            MOVF    0x17,W          ;Coge P1
            ANDLW   0x0F            ;Si es el provider 0 sale
            BTFSC   STATUS,Z        ; Skip if not zero
            GOTO    TX9000          ; zakoncz instrukcje

            MOVLW   0x13            ;Ignorar los ltimos 0x13 bytes recibidos
ins_40_loop SUBWF   0x14,f          
            MOVF    0x14,W
            MOVWF   FSR             ;FSR apunta a la nueva direccin
            MOVLW   0x18            
            SUBWF   FSR,W           ;Los datos recibidos comienzan en 0x18. mira si se ha llegado al principio de los datos
            BTFSS   STATUS,C        
            GOTO    Comp_data       ;si se acabaron los datos, ir a comp_data
            MOVF    INDF,W          ;Lee el valor 
            XORLW   0x90            ; actualizacin de key
            BTFSS   STATUS,Z        ; Skip if zero
            GOTO    TX9000          ; zakoncz instrukcje

            MOVF    FSR,W           ;Es una clave primaria: coge la direccin 
            SUBLW   0x0A            ;Calcula el desplazamiento: 2 bytes tras la direccin anterior (para saltar el 9050)
            MOVWF   EEADR            
            MOVLW   0x0C
            CALL    xor_eedat       ;Swap de 0x0c-0x13 con los datos de la MK
            CALL    xor_24_30       ;Swap de 0x0c-0x13 con 0x30-0x37 
            CALL    xor_2C_38       ;Swap de 0x0c-0x13 con 0x38-0x3f
            CALL    Encrypt         ;Desencripta
            CALL    xor_F8_30       ;Swap de 0x30-0x37 con 0x38-0x3f
            CALL    Comp_key        
            MOVLW   0x0A            ;Saltarse 0x0a bytes 
            GOTO    ins_40_loop
;--------------------------------------------
;Actualiza la fecha de suscripcin
Comp_data   ADDLW   0x1F          ;Puesrto que solo se guardan los ltimos 0x31 bytes de datos el nano 0x21
            MOVWF   FSR           ;est en la posicin 0x1d. Pone 0x1f porque W vale 0x16-0x18 
            MOVF    INDF,W
            XORLW   0x21              ; subscription
            BTFSS   STATUS,Z          ; Skip if zero
            GOTO    TX9000            ; zakoncz instrukcje

            CLRF    PCLATH        ;Datos en pag. 0 de EEPROM   
            MOVLW   0x0E          ;Offset de la primera fecha  
            INCF    0x17,f        ;Suma 1 al num de provider  
Comp_data_loop  
            ADDLW   0x18
            BTFSC   STATUS,C          ; Skip if no carry
            INCF    PCLATH,f
            DECFSZ  0x17,f           ;Bucle que calcula el offset de la fecha
            GOTO    Comp_data_loop
            MOVWF   0x0C
            CALL    i2c_wr            ; Escribe direccin de la EEPROM
            INCF    FSR,f
            CALL    i2c_rd            ;\Lee byte de fecha
            XORWF   INDF,W            ;/
            BTFSS   STATUS,Z          ; Si es igual no lo escribe
            GOTO    Wr_data           ; Si es diferente: actualizar
            INCF    FSR,f
            CALL    i2c_rd            ;\lee segundo byte de fecha
            XORWF   INDF,W            ;/
            BTFSC   STATUS,Z          ; Y mira si ha cambiado
            GOTO    TX9000            ; sino ha cambiado, sale

            DECF    FSR,f         
Wr_data     MOVF    0x0C,W            ;Escribe los dos bytes de la fecha en la EEPROM
            CALL    i2c_wr            ; adres
            MOVF    INDF,W
            CALL    wystaw_byte       ;\
            INCF    FSR,f             ; zapisz nowa date
            MOVF    INDF,W
            CALL    wr_last           ;/
            GOTO    TX9000
;---------------------------------------------
; zapisz klucz jesli nowy
Comp_key    INCF    0x14,W            ;Siguiendo al 90 est el nmero de key que se actualiza 
            MOVWF   FSR
            MOVF    INDF,W            ;Nmero de key (5C, 5D...) el 5 se ignora enla rutina de clculo del offset
            MOVWF   0x0E
            MOVF    0x17,W            ;Provider
            MOVWF   0x0D                   
            CALL    c112c             ; Offset de la key en EEPROM externa
            MOVF    0x10,W
            CALL    i2c_wr            ; Envia direccin
            MOVLW   0x38              ; Donde est almacenada la nueva key
            MOVWF   FSR
Comp_key_loop  
            CALL    i2c_rd            ;Lee un byte
            XORWF   INDF,W
            BTFSS   STATUS,Z          ; Skip if zero
            GOTO    Wr_key            ;Si es diferente: Actualizar clave
            INCF    FSR,f             ;Si es igual: seguir comparando
            BTFSC   FSR,03
            GOTO    Comp_key_loop
            RETURN
;-----------------------------------------------
Wr_key      MOVF    0x10,W            ;Actualiza una clave
            CALL    i2c_wr            ; adres
            MOVLW   0x08              ;\ length
            MOVWF   0x0F              ;/
            CALL    wr_i2c_string
            RETURN
;---------------------------------------
;Swap de bloques 0x30-0x37 y 0x0c-0x13
xor_24_30   MOVLW   0x24
            MOVWF   EEADR
            MOVLW   0x30
            CALL    xor_eedat
            RETURN
;---------------------------------------
;Swap de bloques 0x38-0x3f y 0x0c-0x13
xor_2C_38  MOVLW   0x2C
            MOVWF   EEADR
            MOVLW   0x38
            CALL    xor_eedat
            RETURN
;-------------------------------------------
; koncowka komendy C1 32
;Devuelve un package bitmap
c3A6c       MOVLW   0x83       ; nano 83: sigue un package bitmap
            CALL    Send
            MOVLW   0x0A
            SUBWF   0x0F,W     ;Resta 0x0A a la longitud pedida (1 byte de nano 0x83, 8 de bitmap y 1 de nano 0x04)
            MOVWF   0x10       ;y la almacena en 0x10
            MOVLW   0x08       ;Almacena 0x08 en 0x0f
            MOVWF   0x0F
            BCF     STATUS,C   ; Clear carry flag
            RLF     0x0D,f
            RLF     0x0D,f
            RLF     0x0D,W      ;W=Provider*8  
            INCF    PCLATH,f    ;Estn a aprtir de 0x200 y ocupan 8 bytes cada uno
            CALL    EEPROM_send ;Enva el bitmap 
            MOVLW   0x04                
            CALL    Send        ;Enva nano 0x04
c3a6c_loop  DECF    0x10,f      ;Rellena el resto del paquete con 0xFF
            BTFSC   0x10,07
            GOTO    TX9000
            MOVLW   0xFF
            CALL    Send
            GOTO    c3a6c_loop

;---------------------------------------------
;Lee un bloque de datos de la EEPROM externa
Instr_21    MOVF    0x0D,W       ;P1: Parte alta de la direccin a leer
            ADDWF   PCLATH,f     
            MOVF    0x0E,W       ;P2: Parte baja
            CALL    i2c_wr       ;Enva direccin
instr_21_loop  
            CALL    i2c_rd       ;Lee valor de la EEPROM externa
            CALL    Send         ;Y lo enva  
            DECFSZ  0x0F,f      
            GOTO    instr_21_loop
            GOTO    TX9000       

 ;----------------------------------------------
            ORG     0x2000
            DATA    0x0F
            DATA    0x0F
            DATA    0x0F
            DATA    0x0F

;========================================
; fuse
 
            ORG     0x2007
            DATA    0x19

;========================================
; EEPROM
 
            ORG     0x2100
ATR         DATA    0x00
            DATA    0xD6
            DATA    0xB6
            DATA    0x6C
            DATA    0x0E
            DATA    0x04
            DATA    0x30
            DATA    0x54
            DATA    0x96
            DATA    0x40
            DATA    0x01
            DATA    0x00
            DATA    0x11
            DATA    0xF7
            DATA    0x3B
;========================================
;COMMAND
            DATA    0x0E
            DATA    0x16
            DATA    0x12
            DATA    0x34
            DATA    0x32
            DATA    0x20
            DATA    0x30
            DATA    0x21
            DATA    0x3A
            DATA    0x3C
            DATA    0x40
            DATA    0x1A
            DATA    0x5A
;========================================

            END
